<?php
if(!defined('__IES__'))
{
	define('__IES__', true);

	class IEs extends Storage
	{

		const LABEL_NOGROUP	= ' ';

		function __construct(&$connection)
		{
			parent::__construct($connection);
		}

		function CountAll()
		{
			return $this->CountRecs('ies');
		}

		function CountAllGroups()
		{
			return $this->CountRecs('ie_groups');
		}

		function CountOrdinary()
		{
			return $this->CountRecs('ies', 'Group_Id IS NULL');
		}

		function CountSpecial()
		{
			return $this->CountRecs('ies', 'Group_Id IS NOT NULL');
		}

		function GetNextId()
		{
			return $this->_GetEntityProperties('ies');
		}

		function GetNextGroupId()
		{
			return $this->_GetEntityProperties('ie_groups');
		}

		private function CountFields($id, $preview_only = IE::FIELDLIST_FULL)
		{
			return $this->CountRecs('ie_fields', 'Ie_Id='.$id.($preview_only==IE::FIELDLIST_PREVIEW ? ' AND ShowPreview=1' : ''));
		}

		private function CountTemplates($id)
		{
			return $this->CountRecs('ie_templates', 'Ie_Id='.$id);
		}

		function GetIesIdentifiers()
		{
			return $this->GetPairs('ies', null, 'Identifier');
		}

		/**
		 * IEs::GetAll()
		 *    .
		 *   ,  - ID .
		 *
		 * @param $special     :  (   )   
		 * @param $order
		 * @return array
		 */
		function GetAll($special = false, $order = 'Title')
		{
            $ret = $this->GetAllRecs('ies', '*', 'Ie_Id', $order, 'Group_Id'.( ($special) ? ' IS NOT' : ' IS').' NULL' );
			foreach ($ret as $key => $value)
            {
            	$ret[$key]['FIELDS'] = $this->CountFields($key);
				$ret[$key]['TEMPLATES'] = $this->CountTemplates($key);
            }
			return $ret;
		}

		/**
		 * IEs::GetAllValid()
		 *   , ""  , ..      .
		 *   ,  - ID .
		 *
		 * @param $special      (   )  
		 * @param $order
		 * @return array
		 */
		function GetAllValid($special = false, $order = 'Title')
		{
            $retAll = $this->GetAll($special, $order);
			$ret = Array();
			foreach ($retAll as $key => $value)
            {
				if ($value['FIELDS'] > 0 && $value['TEMPLATES'] > 0)
				{
	                $ret[$key] = $retAll[$key];
                }
            }
			return $ret;
		}

		function GetByGroup($group_id = null)
		{
			return $this->GetAllRecs('ies', 'Identifier,Title', 'Ie_Id', 'Ord, Title', 'Group_Id IS NULL AND (Ie_Group_Id='.(int)$group_id.($group_id==null?' OR Ie_Group_Id IS NULL':'').')');
		}

		function GetGrouped($valid = true)
		{
			$ret = array();
			$ies = $valid ? $this->GetAllValid(false, 'Ord, Title') : $this->GetAll(false, 'Ord, Title');
			$groups = $this->GetAllGroups();
			$groups[0]['Identifier'] = '';
			$groups[0]['Title'] = self::LABEL_NOGROUP;
			$groups[0]['Ord'] = 0;
			foreach ($groups as $gid => $gdata) foreach ($ies as $ieid => $iedata)
			{
				if ($iedata['Ie_Group_Id']==$gid)
				{
					$ret[$gid]['Identifier'] = $gdata['Identifier'];
					$ret[$gid]['Title'] = $gdata['Title'];
					$ret[$gid]['Ord'] = $gdata['Ord'];
					$ret[$gid]['ies'][$ieid] = $iedata;
				}
			}
			return $ret;
		}

		function GetAllGroups()
		{
			return $this->GetAllRecs('ie_groups', '*', 'Ie_Group_Id', 'Ord');
		}

		function GetGroupsIdentifiers()
		{
			return $this->GetPairs('ie_groups', null, 'Identifier');
		}

		function GetGroup($id)
		{
			return $this->GetRec('ie_groups', '*', 'Ie_Group_Id='.(int)$id);
		}

		function CreateGroup($identifier, $title)
		{
			if ($this->CountRecs('ie_groups', 'Identifier=\''.$this->connection->EscapeString($identifier).'\'') > 0) return false;
			$MAX = $this->GetRec('ie_groups', 'MAX(Ord)');
			$sql = 'INSERT INTO ie_groups SET Identifier=\''.$this->connection->EscapeString($identifier).'\', Title=\''.$this->connection->EscapeString($title).'\', Ord='.($MAX+1);
			if ($this->connection->exec($sql))
				return $this->connection->getInsertedId();
			else
				return false;
		}

		function UpdateGroup($id, $title, $ie_ids)
		{
			$sql = 'UPDATE ie_groups SET Title=\''.$this->connection->EscapeString($title).'\' WHERE Ie_Group_Id='.(int)$id;
			$this->connection->exec($sql);
			$sql = 'UPDATE ies SET Ie_Group_Id=NULL, Ord=0 WHERE Ie_Group_Id='.(int)$id;
			$this->connection->exec($sql);
			$ie_ids = explode(',', $ie_ids);
			for ($i=0, $c=count($ie_ids); $i<$c; $i++)
			{
				$sql = 'UPDATE ies SET Ie_Group_Id='.(int)$id.', Ord='.($i+1).' WHERE Ie_Id='.$ie_ids[$i];
				$this->connection->exec($sql);
			}
			return true;
		}

		function DeleteGroup($id)
		{
			$ord = $this->GetRec('ie_groups', 'Ord', 'Ie_Group_Id='.(int)$id);
			$sql = 'DELETE FROM ie_groups WHERE Ie_Group_Id='.(int)$id;
			if ($this->connection->exec($sql))
			{
				$sql = 'UPDATE ie_groups SET Ord=Ord-1 WHERE Ord>'.$ord;
				$this->connection->exec($sql);
				$sql = 'UPDATE ies SET Ie_Group_Id=0, Ord=0 WHERE Ie_Group_Id='.(int)$id;
				$this->connection->exec($sql);
				return true;
			}
			return false;
		}

		function ChangeOrder($id, $inc)
		{
			$ord = $this->GetRec('ie_groups', 'Ord', 'Ie_Group_Id='.(int)$id);
			$new_ord = $ord+$inc;
			if ($inc < 0 && $new_ord < 1) return false;
			else
			{
				$MAX = $this->GetRec('ie_groups', 'MAX(Ord)');
				if ($new_ord > $MAX) return false;
			}
			$sql = 'UPDATE ie_groups SET Ord=Ord-'.(abs($inc)/$inc).' WHERE Ord'.($inc<0?'<':'>').$ord.' AND Ord'.($inc<0?'>=':'<=').$new_ord;
			$this->connection->exec($sql);
			$sql = 'UPDATE ie_groups SET Ord=Ord+'.$inc.' WHERE Ie_Group_Id='.(int)$id;
			$this->connection->exec($sql);
			return true;
		}

		function GetByIdentifier($identifier)
		{
        	$ret = $this->GetRec("ies", "Ie_Id,Title", "Identifier='$identifier' AND Group_Id IS NULL");
            if (is_array($ret))
            {
               	$ret["Identifier"] = $identifier;
                $ret["FIELDS"] = $this->CountFields($ret["Ie_Id"]);
                $ret["TEMPLATES"] = $this->CountTemplates($ret["Ie_Id"]);
            }
            else
            	$ret = false;
			return $ret;
		}

		function GetObjByIdentifier($identifier)
		{
        	$ret = $this->GetRec("ies", "Ie_Id", "Identifier='$identifier'");
            if (!$ret)
            {
            	$ret = false;
			}
            else
			{
				$ret = new IE($this->connection, $ret);
            }
			return $ret;
		}

		function GetPublishedByIeType($ie_type, $rub_id, $page = 0, $page_item_count = DB::LIMIT_CLAUSE_LAST)
		{
        	return $this->GetPairs("rub_ie", null, "Ie", "Pub_Date DESC", "Rub=$rub_id AND Ie_Table=$ie_type", ($page*$page_item_count).",".$page_item_count);
		}

		/**
		 * IEs::GetIesIdsByTemplates()
		 *   ,     .
		 *    ID .
		 *
		 * @param	$templates	-   
		 * @return	array
		 */
		function GetIesIdsByTemplates($templates)
		{
			$ret = array();
			if (!is_array($templates)) $templates = array($templates);
			if (count($templates) > 0)
			{
				foreach ($templates as $template)
				{
					$ret = array_unique(array_merge($ret, $this->GetPairs('ie_templates', null, 'Ie_Id', '1', "Identifier='$template'")));
				}
			}
			return $ret;
		}

		//       
		function GetPublishedIes($ie_type = null, $not_ie_type = null, $page = 0, $page_item_count = DB::LIMIT_CLAUSE_LAST)
		{
			$limit = ($page*$page_item_count).",".$page_item_count;
			if (is_array($ie_type) && count($ie_type)>0)
			{
				if (is_array($not_ie_type) && count($not_ie_type)>0)
				{
					return $this->GetAllRecs("rub_ie", "*", null, "Pub_Date DESC", "Ie_Table IN (".implode(",",$ie_type).") AND Ie_Table NOT IN (".implode(",",$not_ie_type).")", $limit);
				}
				else
				{
					return $this->GetAllRecs("rub_ie", "*", null, "Pub_Date DESC", "Ie_Table IN (".implode(",",$ie_type).") AND ".(($not_ie_type!=null) ? "Ie_Table<>".(int)$not_ie_type : DB::WHERE_CLAUSE_NEUTRAL), $limit);
				}
			}
			else
			{
				if (is_array($not_ie_type) && count($not_ie_type)>0)
				{
					return $this->GetAllRecs("rub_ie", "*", null, "Pub_Date DESC", (($ie_type!=null) ? "Ie_Table=".(int)$ie_type : DB::WHERE_CLAUSE_NEUTRAL)." AND Ie_Table NOT IN (".implode(",",$not_ie_type).")", $limit);
				}
				else
				{
					return $this->GetAllRecs("rub_ie", "*", null, "Pub_Date DESC", (($ie_type!=null) ? "Ie_Table=".(int)$ie_type : DB::WHERE_CLAUSE_NEUTRAL)." AND ".(($not_ie_type!=null) ? "Ie_Table<>".(int)$not_ie_type : DB::WHERE_CLAUSE_NEUTRAL) , $limit);
				}
			}
		}
	}
}
?>