<?php
if(!defined('__RUB__'))
{
	define('__RUB__', true);

	class Rub extends Storage
	{
		const SORT_IE_PUBLDATE	= 0;
		const SORT_IE_HAND 		= 1;
		const SORT_IE_ALPHABET	= 2;
		
		const FLAG_LOCKDELETE	= 1; //  
		const FLAG_LOCKIDENT	= 2; //  
		const FLAG_LOCKPUBLISH	= 4; //    
		
		public $identifier	= null;
		public $group_id	= null;
		public $rub_parent	= null;
		public $ord			= null;
		public $ie_data_id	= null;
		public $sorttype	= null;
		public $title		= '';
		public $flags		= 0;

		function __construct(&$connection, $id = null)
		{
			parent::__construct($connection, 'rubs', 'Rub_Id');
			$this->SetProperties($id);
		}

		function GetDataId()
		{
			if ($this->id == null) return false;
			return $this->ie_data_id;
		}

		function GetNextId()
		{
			return $this->_GetEntityProperties($this->table, 'Auto_increment');
		}

		function GetNicePath($path, $separator='/')
		{
			if ($this->id == null) return false;
			$path = $this->GetParam('identifier') . $separator . $path;
			if ($this->rub_parent != 0)
			{
				$rub = new Rub($this->connection, $this->rub_parent);
				$path = $rub->GetNicePath($path, $separator);
			}
			return rtrim($path,$separator);
		}

		function SetOrder($ord, $old_rub_parent)
		{
			if ($this->id == null) return false;
            $MAX = $this->GetRec("rubs", "MAX(Ord)", "Group_Id=".$this->group_id." AND Rub_Parent=".$this->rub_parent);
			if ($this->ord != null)
			{
				//modify
				$sql = "UPDATE rubs SET Ord=Ord-1 WHERE Group_Id=".$this->group_id." AND Ord>".$this->ord." AND Rub_Parent=".$old_rub_parent;
				$this->connection->exec($sql);
			}
			$MAX = $MAX<1 ? 1 : $MAX+1;
			$ord = ($ord>$MAX || $ord<1) ? $MAX : (int)$ord;
			$sql = "UPDATE rubs SET Ord=Ord+1 WHERE Group_Id=".$this->group_id." AND Ord>=".$ord." AND Rub_Parent=".$this->rub_parent;
			$this->connection->exec($sql);
			$sql = "UPDATE rubs SET Ord=".$ord." WHERE Rub_Id=".$this->id;
			$this->connection->exec($sql);
			return $ord;
		}

		/**
		 * Rub::checkIdentifier()
		 *
		 * @return  boolean
		 * TRUE            
		 *        .
		 */
		function checkIdentifier($identifier, $group_id, $rub_parent)
		{
			$where = "Identifier='$identifier' AND Group_Id=$group_id AND Rub_Parent=$rub_parent";
			if ($this->id != null) $where .= ' AND Rub_Id<>'.$this->id;
			if ($this->CountRecs($this->table, $where) > 0)
				return true;
			else
				return false;
		}

		function Create($title, $group_id, $rub_parent, $ord, $identifier, $data_id, $sorttype = self::SORT_IE_PUBLDATE, $flags = 0)
		{
			if ($this->checkIdentifier($identifier, $group_id, $rub_parent))
			{
				//$_SESSION["Error"]->Add(array("level"=>E_ERROR, "message"=>"       . [Rub::Create()]"));
				return false;
			}
			$sql = "INSERT INTO rubs SET Identifier='$identifier', Title='$title', Group_Id=$group_id, Rub_Parent=$rub_parent, Ie_Data_Id=$data_id, SortType=$sorttype, Flags=$flags";
			if ($this->connection->exec($sql))
			{
				$this->id = $this->connection->getInsertedId();
				$this->identifier = $identifier;
				$this->sorttype = $sorttype;
				$this->rub_parent = $rub_parent;
				$this->group_id = $group_id;
				$this->ord = $this->SetOrder($ord, $this->rub_parent);
                $this->ie_data_id = $data_id;
				$this->title = $title;
				$this->flags = $flags;
				return $this->id;
			}
			else return false;
		}

		function GetChildrenIds()
		{
			if ($this->id == null) return false;
			return $this->GetPairs("rubs", null, "Rub_Id", "Ord", "Rub_Parent=".$this->id);
		}

		function GetChildren()
		{
			if ($this->id == null) return false;
			$c = array();
            $ids = $this->GetChildrenIds();
            foreach ($ids as $id)
			{
                array_push($c, new Rub($this->connection, $id));
			}
			return $c;
		}

		function GetSiblings()
		{
			if ($this->id == null) return false;
			$n = array();
            $ids = $this->GetPairs("rubs", null, "Rub_Id", "Ord", "Rub_Parent=".$this->rub_parent." AND Group_Id=".$this->group_id);
            foreach ($ids as $id)
			{
				array_push($n, ($id==$this->id ? $this : new Rub($this->connection, $id)));
			}
			return $n;
		}

		function GetChildrenTreeList(&$c, $id)
		{
            $ids = $this->GetPairs("rubs", null, "Rub_Id", "Ord", "Rub_Parent=".$id);
            foreach ($ids as $id)
			{
				array_push($c, $id);
				$this->GetChildrenTreeList($c, $id);
			}
		}

		/**
		 * private Rubricator::checkChildren()
		 *
		 * @return  boolean
		 * TRUE       
		 */
		function CheckChildren()
		{
			if ($this->id == null) return false;
			$children = array();
			$this->GetChildrenTreeList($children, $this->id);
			return in_array($this->id, $children);
		}

		function Update($title, $rub_parent, $ord, $identifier, $sorttype = self::SORT_IE_PUBLDATE, $flags = 0)
		{
			if ($this->id == null) return false;
			//    
			if ($this->checkIdentifier($identifier, $this->group_id, $rub_parent))
			{
				//$_SESSION["Error"]->Add(array("level"=>E_ERROR, "message"=>"       . [Rub::Update()]"));
				return false;
			}
			//  
			if ($this->CheckChildren())
			{
				$_SESSION["Error"]->Add(array("level"=>E_ERROR, "message"=>"      [Rub::Update()]"));
				return false;
			}
			$sql = "UPDATE rubs SET Identifier='$identifier', Title='$title', Rub_Parent=$rub_parent, SortType=$sorttype, Ord=0, Flags=$flags WHERE Rub_Id=".$this->id;
			$return = $this->connection->exec($sql);
            if ($return)
            {
				$old_rub_parent = $this->rub_parent;
				$this->rub_parent = $rub_parent;
				$this->identifier = $identifier;
				$this->sorttype = $sorttype;
				$this->ord = $this->SetOrder($ord, $old_rub_parent);
				$this->title = $title;
            }
			return $return;
		}

		function Delete()
		{
			if ($this->id == null) return false;
			if (count($this->GetChildrenIds()) > 0) return false; // error. can not delete rubric with subrubrics
			if ($this->GetPublishedCount() > 0) return false; // error can not delete rubric with published ies
			$sql = "DELETE FROM publish_permissions WHERE Rub_Id=".$this->id;
			$this->connection->exec($sql);
			$sql = "DELETE FROM rubs WHERE Rub_Id=".$this->id;
			$this->connection->exec($sql);
			$sql = "UPDATE rubs SET Ord=Ord-1 WHERE Group_Id=".$this->group_id." AND Rub_Parent=".$this->rub_parent." AND Ord>".$this->ord;
			$this->connection->exec($sql);
			$rubricator = new Rubricator($this->connection, $this->group_id);
			$ie = new IE($this->connection, $rubricator->GetIeId());
			$ie->DeleteData($this->ie_data_id);
			$this->id = null;
			$this->identifier = null;
			$this->sorttype = null;
			$this->group_id = null;
			$this->rub_parent = null;
			$this->ord = null;
			$this->ie_data_id = null;
			$this->flags = null;
			return true;
		}

		function PublishIe($ie_type_id, $ie_id, $admin_id=0)
		{
			if ($this->id == null) return false;
			$ie = new IE($this->connection, $ie_type_id);
			return $ie->PublishInRubs($ie_id, array($this->id), $admin_id);
		}

		function UnpublishIe($ie_type_id, $ie_id)
		{
			if ($this->id == null) return false;
			$ord = (int)$this->GetRec("rub_ie", "MAX(Ord)", "Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id);
			$sql = "DELETE FROM rub_ie WHERE Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id;
			$this->connection->exec($sql);
			$sql = "UPDATE rub_ie SET Ord=Ord-1 WHERE Rub=".$this->id." AND Ord>".$ord;
			return $this->connection->exec($sql);
		}

		function UnpublishAllIe()
		{
			if ($this->id == null) return false;
			$sql = "DELETE FROM rub_ie WHERE Rub=".$this->id;
			return $this->connection->exec($sql);
		}

		function MovePublicationDown($ie_type_id, $ie_id)
		{
			if ($this->id == null) return false;
			$ord = (int)$this->GetRec("rub_ie", "MAX(Ord)", "Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id) + 1;
			$sql = "UPDATE rub_ie SET Ord=Ord-1 WHERE Ord=".$ord." AND Rub=".$this->id;
			$res = $this->connection->exec($sql);
			$sql = "UPDATE rub_ie SET Ord=Ord+1 WHERE Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id;
			$res = $this->connection->exec($sql);
		}

		function MovePublicationUp($ie_type_id, $ie_id)
		{
			if ($this->id == null) return false;
			$ord = (int)$this->GetRec("rub_ie", "MAX(Ord)", "Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id) - 1;
			$sql = "UPDATE rub_ie SET Ord=Ord+1 WHERE Ord=".$ord." AND Rub=".$this->id;
			$res = $this->connection->exec($sql);
			$sql = "UPDATE rub_ie SET Ord=Ord-1 WHERE Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id;
			$res = $this->connection->exec($sql);
		}

		function MovePublicationTop($ie_type_id, $ie_id)
		{
			if ($this->id == null) return false;
			$ord = (int)$this->GetRec("rub_ie", "MAX(Ord)", "Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id);
			$sql = "UPDATE rub_ie SET Ord=Ord+1 WHERE Ord<".$ord." AND Rub=".$this->id;
			$res = $this->connection->exec($sql);
			$sql = "UPDATE rub_ie SET Ord=1 WHERE Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id;
			$res = $this->connection->exec($sql);
		}

		function MovePublicationBottom($ie_type_id, $ie_id)
		{
			if ($this->id == null) return false;
			$ord = (int)$this->GetRec("rub_ie", "MAX(Ord)", "Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id);
			$max = (int)$this->GetRec("rub_ie", "COUNT(*)", "Rub=".$this->id);
			$sql = "UPDATE rub_ie SET Ord=Ord-1 WHERE Ord>".$ord." AND Rub=".$this->id;
			$res = $this->connection->exec($sql);
			$sql = "UPDATE rub_ie SET Ord=".$max." WHERE Rub=".$this->id." AND Ie_Table=".(int)$ie_type_id." AND Ie=".(int)$ie_id;
			$res = $this->connection->exec($sql);
		}

		function GetPath($path = array())
		{
			if ($this->id == null) return false;
			$path[] = get_object_vars($this);
			if ($this->rub_parent != 0)
			{
				$rub = $this->GetParent();
				$path = $rub->GetPath($path);
			}
			return $path;
		}

		function GetParent()
		{
			if ($this->rub_parent == 0) return 0;
			return new Rub($this->connection, $this->rub_parent);
		}

		function GetPublishAdmin($ie_id, $id)
		{
			if ($this->id == null) return false;
			return $this->GetRec("rub_ie", "Admin_Id", "Rub=".$this->id." AND Ie=".$id." AND Ie_Table=".$ie_id);
		}

		function GetData($identifiers = array(), &$rubricator = null)
		{
			if ($this->id == null) return false;
			if ($rubricator == null) $rubricator = new Rubricator($this->connection, $this->group_id);
			return $rubricator->GetData($this->ie_data_id, $identifiers);
		}

		function GetPublicationRange()
		{
			if ($this->id == null) return false;
			return array(
				$this->GetRec('rub_ie', 'MIN(Pub_Date)', 'Rub='.$this->id),
				$this->GetRec('rub_ie', 'MAX(Pub_Date)', 'Rub='.$this->id),
			);
		}

		function CheckIEPublished($ie_type, $ie_id)
		{
			if ($this->id == null) return false;
			return $this->GetRec('rub_ie', '*', 'Rub='.$this->id.' AND Ie_Table='.$ie_type.' AND Ie='.$ie_id);
		}

		function GetPublished($page = 0, $page_item_count = DB::LIMIT_CLAUSE_LAST, $ie_type = array(), $alpha = '', $reverse_ord = false)
		{
			if ($this->id == null) return false;
			switch($this->sorttype)
			{
				case self::SORT_IE_PUBLDATE:
					$sortstring = 'Pub_Date'.($reverse_ord?' ASC':' DESC');
				break;
				case self::SORT_IE_HAND:
					$sortstring = 'Ord'.($reverse_ord?' DESC':' ASC');
				break;
				case self::SORT_IE_ALPHABET:
					$sortstring = 'Sort_String'.($reverse_ord?' DESC':' ASC');
				break;
			}
			return $this->GetAllRecs(
						'rub_ie'
						, 'SQL_CALC_FOUND_ROWS *'
						, null
						, $sortstring
						, 'Rub='.$this->GetId()
						  .(count($ie_type)>0?' AND Ie_Table IN ('.implode(',', $ie_type).')':'')
						  .($alpha!=''?" AND UPPER(LEFT(Sort_String, 1))='$alpha'":'')
						, ($page*$page_item_count).','.$page_item_count
						);
		}
		
		function CalcFoundRows()
		{
			$res = $this->connection->exec('SELECT FOUND_ROWS()');
			return $this->connection->getField($res);
		}

		function GetPublishedRandom($count = 1)
		{
			if ($this->id == null) return false;
			if ($count < 1) $count = 1;
			return $this->GetAllRecs(
						'rub_ie'
						, '*'
						, null
						, 'RAND()'
						, 'Rub='.$this->GetId()
						, '0,'.$count
						);
		}

		function GetPublishedCount($ie_type = array(), $alpha = '')
		{
			if ($this->id == null) return false;
			return $this->CountRecs(
						'rub_ie'
						, 'Rub='.$this->GetId()
						  .(count($ie_type)>0?' AND Ie_Table IN ('.implode(',', $ie_type).')':'')
						  .($alpha!=''?" AND UPPER(LEFT(Sort_String, 1))='$alpha'":'')
						);
		}

		//    
		function GetPublishedByDate($date_start, $date_stop, $page = 0, $page_item_count = DB::LIMIT_CLAUSE_LAST)
		{
			if ($date_start==null && $date_stop==null) return false;
			return $this->GetAllRecs('rub_ie', '*', null, 'Pub_Date', DB::WHERE_CLAUSE_NEUTRAL.($this->id!=null?' AND Rub='.$this->id:'').($date_start!=null?" AND Pub_Date>='".$date_start."'":'').($date_stop!=null?" AND Pub_Date<='".$date_stop."'":''), ($page*$page_item_count).",".$page_item_count);
		}
		
		function GetPublishedCountByDate($date_start, $date_stop)
		{
			return $this->CountRecs('rub_ie', DB::WHERE_CLAUSE_NEUTRAL.($this->id!=null?' AND Rub='.$this->id:'').($date_start!=null?" AND Pub_Date>='".$date_start."'":'').($date_stop!=null?" AND Pub_Date<='".$date_stop."'":''));
		}

		function GetAlphabet()
		{
			if ($this->id == null) return false;
			return $this->GetPairs(
				'rub_ie', 
				null,
				'DISTINCT UPPER(LEFT(Sort_String, 1)) LET', 
				'LET',
				'Rub='.$this->id);
		}
		
		function GetPublishedByOrder($ord)
		{
			if ($this->id == null) return false;
			return $this->GetRec('rub_ie', '*', 'Rub='.$this->id.' AND `Ord`='.(int)$ord);
		}
	}
}
?>