<?php
if(!defined('__CACHE__'))
{
	define('__CACHE__', true);
	
	if (!extension_loaded('memcache'))
	{
		error_log(':    "Memcache"');
		class Cache
		{
			function IsConnected()	{return false;}
			function GetKeys()		{return array();}
			function get()			{return false;}
			function add()			{return false;}
			function set()			{return false;}
			function delete()		{return false;}
			function getStats()		{return array();}
			function getVersion()	{return 'error';}
			function dropKey()		{return false;}
			function Debug()		{}
			function flush()		{return false;}
		}
		return;
	}
	class Cache extends Memcache
	{
		const MEMCACHE_STOREDKEYS	= 'MemcacheStoredKeys';
		const MEMCACHE_FLAG			= 0;
		const MEMCACHE_EXPIRE_MAX	= 2592000;
		
		const DEBUG_ECHO_KEYS		= 1;
		
		protected $keys			= array();
		protected $host			= 'localhost';
		protected $port			= null; // 11211
		protected $isConnected	= false;
		protected $debug		= false;
		protected $debug_level	= 0;
		
		function __construct($host = null, $port = null)
		{
			if ($host != null) $this->host = $host;
			if ($port != null) $this->port = $port;
			if ($this->port != null)
				$this->isConnected = @$this->connect($this->host, $this->port);
			else
				$this->isConnected = @$this->connect($this->host);
			$this->initKeys();
		}
		
		function Debug($debug_level = 0)
		{
			$this->debug = true;
			$this->debug_level = $debug_level;
		}
		
		function get($key)
		{
			if ($this->debug)
			{
				if ($this->debug_level == self::DEBUG_ECHO_KEYS)
				{
					echo " [ ";
					if (parent::get($key)) echo '+'; else echo '-';
					echo "$key ] ";
				}
				return false;
			}
			return parent::get($key);
		}
		
		function IsConnected()
		{
			return $this->isConnected;
		}
		
		protected function initKeys()
		{
			if (!$this->isConnected) return false;
			parent::add(self::MEMCACHE_STOREDKEYS, $this->keys, self::MEMCACHE_FLAG);
			$this->keys = parent::get(self::MEMCACHE_STOREDKEYS);
			return true;
		}
		
		protected function saveKeys()
		{
			if (!$this->isConnected) return false;
			return parent::set(self::MEMCACHE_STOREDKEYS, $this->keys, self::MEMCACHE_FLAG);
		}
		
		function GetKeys()
		{
			if (!$this->isConnected) return false;
			return $this->keys;
		}
		
		protected function setKey($key, $title, $expire, $obit)
		{
			if (!$this->isConnected) return false;
			$this->keys[$key] = array('title'=>$title, 'time'=>$expire, 'obit'=>$obit); // obit -    .
			return $this->saveKeys();
		}
		
		function dropKey($key)
		{
			if (!$this->isConnected) return false;
			//unset($this->keys[$key]);
			$this->keys[$key]['dropped']=true;
			return $this->saveKeys();
		}
		
		function add($key, $var, $title, $expire = 0, $flag = null)
		{
			if (!$this->isConnected) return false;
			if ($this->debug) return false;
			if (!parent::add($key, $var, $flag, $expire)) return false;
			if ($title != '') return $this->setKey($key, $title, $expire);
			return true;
		}
		
		function set($key, $var, $title='', $expire = 0, $flag = null)
		{
			if (!$this->isConnected) return false;
			if ($this->debug) return false;
			if ($expire == 0   ||   $expire > self::MEMCACHE_EXPIRE_MAX && $expire <= time()) { $expire = 0; $obit = time()+self::MEMCACHE_EXPIRE_MAX; }
			else if ($expire > time()) { $obit = $expire; $expire = $expire-time(); }
			else { $obit = time()+$expire; }
			if (!parent::set($key, $var, $flag, $expire)) return false;
			if ($title != '') return $this->setKey($key, $title, $expire, $obit);
			return true;
		}
		
		function delete($key, $timeout=0)
		{
			if (!$this->isConnected) return false;
			if (!parent::delete($key, $timeout)) return false;
			return $this->dropKey($key);
		}
	}
}
?>