<?php
if(!defined('__SEARCH__'))
{
	define('__SEARCH__', true);

	class Search
	{
		const TYPE_RUB	= 'rub';
		const TYPE_IE	= 'ie';
		public $connection = null;
		public $rubricators = Array();
		public $ies = Array(); // array(id=>int, type=>string, fields=>array)
		function Search(&$connection)
		{
			$this->connection = $connection;
		}

		function GetConnection()
		{
			return $this->connection;
		}

		/**
		 * Search::AddRubricatorByIdentifier()
		 *          IE.
		 *
		 * @param	$rubricator_identifier	array	-  
		 * @param	$ex_fields				array	-         
		 * @return	boolean		TRUE on success
		 */
		function AddRubricatorByIdentifier($rubricator_identifier, $ex_fields=array())
		{
			$this->rubricatorsObj = new Rubricators($this->connection);
			if (!$rubricator = $this->rubricatorsObj->GetByIdentifier($rubricator_identifier))
				return false;
			else
			{
				$this->rubricators[] = Array
				(
					'rubricator' => $rubricator
				);
				$fields = $this->AddIEFields($rubricator->ie_id, $ex_fields);
				if (is_array($fields) && count($fields) > 0)
				{
					$this->ies[] = Array
					(
						'rubricator' => $rubricator->GetId(),
						'id' => $rubricator->ie_id,
						'type' => self::TYPE_RUB,
						'title' => $rubricator->title,
						'fields' => $fields
					);
				}
				return true;
			}
		}

		/**
		 * Search::AddIEFields()
		 *     .
		 * @param	$ie_type			numeric	- IE type id
		 * 			OR					string	- IE type identifier
		 * @param	$ex_fields	array	 -
		 * @return	array	 
		 */
		function AddIEFields($ie_type, $ex_fields=array())
		{
			$ie = new IE($this->connection, $ie_type);
			if (!$ie->GetId()) return false;
			$this->last_ie = $ie;
			$fields = array_diff(
						array_merge(
								$ie->GetFieldsIdentifiersByKind('text')
								, $ie->GetFieldsIdentifiersByKind('html')
								, $ie->GetFieldsIdentifiersByKind('textarea'))
						, $ex_fields);
			if (count($fields) > 0)	return array_values($fields);
			else					return false;
		}

		/**
		 * Search::AddIEByIdentifier()
		 *        .
		 *
		 * @param	$ie_identifier	string	-   
		 * @param	$ex_fields		array	-         
		 * @return	boolean			TRUE on success
		 */
		function AddIEByIdentifier($ie_identifier, $ex_fields=array())
		{
			if (!$fields = $this->AddIEFields($ie_identifier, $ex_fields))
				return false;
			else
			{
				$this->ies[] = Array
				(
					'id' => $this->last_ie->id,
					'type' => self::TYPE_IE,
					'title' => $this->last_ie->title,
					'fields' => $fields
				);
				return true;
			}
		}

		/**
		 * Search::AddAllIEs()
		 *      .
		 *
		 * @param $except -  
		 * @return nothing
		 */
		function AddAllIEs($except = array())
		{
			$ret = false;
			$this->iesObj = new IEs($this->connection);
			foreach($this->iesObj->GetAllValid() as $ie_id => $attrs)
			{
				if (in_array($attrs['Identifier'], $except)) continue;
				$fields = $this->AddIEFields($ie_id);
				if (count($fields) > 0)
				{
					$this->ies[] = Array
					(
						'id' => $ie_id,
						'type' => self::TYPE_IE,
						'title' => $this->last_ie->title,
						'fields' => $fields
					);
					$ret = true;
				}
			}
			return $ret;
		}

		/**
		 * Search::Get()
		 *          ,     .
		 * !
		 *  ,    ,        3, ..       3  .
		 * -  ,     50%    .
		 *
		 * @param	$searchstr	string -  
		 * @return	array	    ,   .
							'type' => ie|rub, see below
								ie
									'rubricator' => rubricator Id where published,
									'rub' => Rub Id where published,
									'ie' => ie id,
									'data' => data id,
									'text' => first encountered field text,
								rub
									'rubricator' => rubricator Id,
									'rub' => Rub Id,
									'ie' => rubricator ie id, (verbose)
									'data' => rubric data id, (verbose)
									'text' => Rub Title,
							'score' => rank
		 */
		function Get($searchstr)
		{
			$ret = Array();
			$searchstr = $this->connection->EscapeString($searchstr);
			foreach($this->ies as $ie)
			{
				$count = count($ie['fields']);
				$searchresults = $fields = Array();
				for($i=0; $i<$count; $i++)
				{
					$fields[$i] = 'MATCH (`'.$ie['fields'][$i]."`) AGAINST ('".$searchstr."')";
				}
				$sql = 'SELECT Id,'.($ie['type']==self::TYPE_IE?$ie['fields'][0].',':'NULL,').'('.implode(')+(', $fields).') as score FROM ie'.$ie['id'].' WHERE '.implode(' OR ', $fields).' HAVING score >= 1';
				$res = $this->connection->exec($sql);
				while($row = $this->connection->fetch($res))
				{
					$searchresults[$row['Id']] = array($row[1], $row['score']);
				}
				if (count($searchresults) == 0) continue;
				switch ($ie['type'])
				{
					case self::TYPE_IE:
						foreach ($searchresults as $data_id => $sr)
						{
							foreach($this->rubricators as $rubricator)
							{
								$sql = 'SELECT * FROM rub_ie RI, rubs R WHERE RI.Ie='.$data_id.' AND RI.Ie_Table='.$ie['id'].' AND R.Rub_Id=RI.Rub AND R.Group_Id='.$rubricator['rubricator']->GetId().' LIMIT 1';
								$res = $this->connection->exec($sql);
								while($row = $this->connection->fetch($res))
								{
									$ret[] = Array(
										'type' => $ie['type'],
										'rubricator' => $rubricator['rubricator']->GetId(),
										'rub' => $row['Rub'],
										'ord' => $row['Ord'],
										'ie' => $ie['id'],
										'data' => $row['Ie'],
										'text' => $sr[0],
										'score' => $sr[1]
									);
								}
							}
						}
					break;
					case self::TYPE_RUB:
						foreach ($searchresults as $data_id => $sr)
						{
							$sql = 'SELECT Rub_Id, Title FROM rubs WHERE Ie_Data_Id='.$data_id.' AND Group_Id='.$ie['rubricator'];
							$row = $this->connection->fetch($this->connection->exec($sql));
							$ret[] = Array(
								'type' => $ie['type'],
								'rubricator' => $ie['rubricator'],
								'rub' => $row['Rub_Id'],
								'ie' => $ie['id'],
								'data' => $data_id,
								'text' => $row['Title'],
								'score' => $searchresults[$data_id][1]
							);
						}
					break;
				}
			}
			function cmpscore($a, $b)
			{
			    if ($a['score'] == $b['score']) return 0;
			    return ($a['score'] > $b['score']) ? -1 : 1;
			}
			usort($ret, 'cmpscore');
			return $ret;
		}
	}
}
?>